@@ -46,10 +46,8 @@ from __future__ import absolute_import, division, print_function
 
 import numbers
 
-try:
-    from pymongo.collection import Collection
-except ImportError:
-    Collection = type(None)
+from pymongo.collection import Collection
+from pymongo.database import Database
 
 import fnmatch
 from datashape.predicates import isscalar
@@ -57,27 +55,33 @@ from toolz import pluck, first, get
 import toolz
 import datetime
 
-from ..expr import (var, Label, std, Sort, count, nunique, nelements, Selection,
+from ..expr import (Sort, count, nunique, nelements, Selection,
                     mean, Reduction, Head, ReLabel, Distinct, ElemWise, By,
                     Symbol, Projection, Field, sum, min, max, Gt, Lt, Ge, Le,
                     Eq, Ne, And, Or, Summary, Like, Broadcast, DateTime,
                     Microsecond, Date, Time, Expr, symbol, Arithmetic, floor,
                     ceil, FloorDiv)
+from ..expr.broadcast import broadcast_collect
 from ..expr import math
 from ..expr.datetime import Day, Month, Year, Minute, Second, UTCFromTimestamp
 from ..compatibility import _strtypes
-from .core import compute
 
 from ..dispatch import dispatch
 
 
 __all__ = ['MongoQuery']
 
+
 @dispatch(Expr, Collection)
 def pre_compute(expr, data, scope=None, **kwargs):
     return MongoQuery(data, [])
 
 
+@dispatch(Expr, Database)
+def pre_compute(expr, data, **kwargs):
+    return data
+
+
 class MongoQuery(object):
     """
     A Pair of a pymongo collection and a aggregate query
@@ -114,11 +118,9 @@ class MongoQuery(object):
         return hash((type(self), self.info()))
 
 
-from ..expr.broadcast import broadcast_collect
-
 @dispatch(Expr, (MongoQuery, Collection))
 def optimize(expr, seq):
-    return broadcast_collect( expr)
+    return broadcast_collect(expr)
 
 
 @dispatch(Head, MongoQuery)
@@ -347,6 +349,11 @@ datetime_terms = {Day: 'dayOfMonth',
                   Second: 'second'}
 
 
+@dispatch(Field, Database)
+def compute_up(expr, data, **kwargs):
+    return getattr(data, expr._name)
+
+
 @dispatch(Expr, Collection)
 def post_compute(e, c, scope=None):
     """
@@ -355,6 +362,13 @@ def post_compute(e, c, scope=None):
     return post_compute(e, MongoQuery(c, ()), scope=scope)
 
 
+def get_result(result):
+    try:
+        return result['result']
+    except TypeError:
+        return list(result)
+
+
 @dispatch(Expr, MongoQuery)
 def post_compute(e, q, scope=None):
     """
@@ -371,13 +385,13 @@ def post_compute(e, q, scope=None):
     q = q.append(scope)
 
     if not e.dshape.shape:  # not a collection
-        result = q.coll.aggregate(list(q.query))['result'][0]
+        result = get_result(q.coll.aggregate(list(q.query)))[0]
         if isscalar(e.dshape.measure):
             return result[e._name]
         else:
             return get(e.fields, result)
 
-    dicts = q.coll.aggregate(list(q.query))['result']
+    dicts = get_result(q.coll.aggregate(list(q.query)))
 
     if isscalar(e.dshape.measure):
         return list(pluck(e.fields[0], dicts, default=None))  # dicts -> values
@@ -394,7 +408,7 @@ def post_compute(e, q, scope=None):
     scope = {'$project': toolz.merge({'_id': 0},  # remove mongo identifier
                                  dict((col, 1) for col in columns))}
     q = q.append(scope)
-    dicts = q.coll.aggregate(list(q.query))['result']
+    dicts = get_result(q.coll.aggregate(list(q.query)))
 
     assert len(columns) == 1
     return list(pluck(first(columns.keys()), dicts))
