@@ -33,8 +33,9 @@ from ..dispatch import dispatch
 from ..expr import (Projection, Field, Sort, Head, Broadcast, Selection,
                     Reduction, Distinct, Join, By, Summary, Label, ReLabel,
                     Map, Apply, Merge, std, var, Like, Slice, summary,
-                    ElemWise, DateTime, Millisecond, Expr, Symbol,
-                    UTCFromTimestamp, nelements, DateTimeTruncate, count)
+                    ElemWise, DateTime, Millisecond, Expr, Symbol, IsIn,
+                    UTCFromTimestamp, nelements, DateTimeTruncate, count,
+                    UnaryStringFunction, nunique)
 from ..expr import UnaryOp, BinOp
 from ..expr import symbol, common_subexpression
 from .core import compute, compute_up, base
@@ -159,6 +160,22 @@ def compute_up(t, df, **kwargs):
     return df.drop_duplicates().reset_index(drop=True)
 
 
+@dispatch(nunique, DataFrame)
+def compute_up(expr, data, **kwargs):
+    return compute_up(expr._child.distinct().count(), data, **kwargs)
+
+
+string_func_names = {
+    'strlen': 'len',
+}
+
+
+@dispatch(UnaryStringFunction, Series)
+def compute_up(expr, data, **kwargs):
+    name = type(expr).__name__
+    return getattr(data.str, string_func_names.get(name, name))()
+
+
 def unpack(seq):
     """ Unpack sequence of length one
 
@@ -534,3 +551,8 @@ units_map = {
 @dispatch(DateTimeTruncate, Series)
 def compute_up(expr, data, **kwargs):
     return Series(compute_up(expr, into(np.ndarray, data), **kwargs))
+
+
+@dispatch(IsIn, Series)
+def compute_up(expr, data, **kwargs):
+    return data.isin(expr._keys)
