@@ -16,32 +16,47 @@ WHERE accounts.amount < :amount_1
 """
 from __future__ import absolute_import, division, print_function
 
-import operator
+import itertools
+from itertools import chain
+
+from operator import and_, eq
+from copy import copy
+
 import sqlalchemy as sa
-import sqlalchemy
+
 from sqlalchemy import sql, Table, MetaData
 from sqlalchemy.sql import Selectable, Select
+from sqlalchemy.ext.compiler import compiles
 from sqlalchemy.sql.elements import ClauseElement, ColumnElement
 from sqlalchemy.engine import Engine
-from operator import and_, eq
-import itertools
-from copy import copy
+
 import toolz
+
+from toolz import unique, concat, pipe
+from toolz.curried import map
+
 from multipledispatch import MDNotImplementedError
-from datashape.predicates import isscalar, isrecord
+
 from odo.backends.sql import metadata_of_engine
 
 from ..dispatch import dispatch
-from ..expr import Projection, Selection, Field, Broadcast, Expr, Symbol
-from ..expr import BinOp, UnaryOp, USub, Join, mean, var, std, Reduction, count
+
+from .core import compute_up, compute, base
+
+from ..expr import Projection, Selection, Field, Broadcast, Expr, IsIn
+from ..expr import (BinOp, UnaryOp, USub, Join, mean, var, std, Reduction,
+                    count, FloorDiv, UnaryStringFunction, strlen, DateTime)
 from ..expr import nunique, Distinct, By, Sort, Head, Label, ReLabel, Merge
 from ..expr import common_subexpression, Summary, Like, nelements
+
+from ..expr.broadcast import broadcast_collect
+
 from ..compatibility import reduce
-from .core import compute_up, compute, base
+
 from ..utils import listpack
 
-__all__ = ['sqlalchemy', 'select']
 
+__all__ = ['sa', 'select']
 
 
 def inner_columns(s):
@@ -51,6 +66,7 @@ def inner_columns(s):
         return s.c
     raise TypeError()
 
+
 @dispatch(Projection, Selectable)
 def compute_up(t, s, scope=None, **kwargs):
     d = dict((c.name, c) for c in inner_columns(s))
@@ -73,7 +89,8 @@ def compute_up(t, s, **kwargs):
 def compute_up(t, s, **kwargs):
     d = dict((t._scalars[0][c], list(inner_columns(s))[i])
              for i, c in enumerate(t._scalars[0].fields))
-    result = compute(t._scalar_expr, d, post_compute=False)
+    name = t._scalar_expr._name
+    result = compute(t._scalar_expr, d, post_compute=False).label(name)
 
     s = copy(s)
     s.append_column(result)
@@ -84,7 +101,14 @@ def compute_up(t, s, **kwargs):
 def compute_up(t, s, **kwargs):
     d = dict((t._scalars[0][c], list(inner_columns(s))[i])
              for i, c in enumerate(t._scalars[0].fields))
-    return compute(t._scalar_expr, d, post_compute=False)
+    name = t._scalar_expr._name
+    return compute(t._scalar_expr, d, post_compute=False).label(name)
+
+
+@dispatch(Broadcast, sa.Column)
+def compute_up(t, s, **kwargs):
+    expr = t._scalar_expr
+    return compute(expr, s, post_compute=False).label(expr._name)
 
 
 @dispatch(BinOp, ColumnElement)
@@ -95,14 +119,24 @@ def compute_up(t, data, **kwargs):
         return t.op(t.lhs, data)
 
 
-@dispatch(BinOp, ColumnElement, (ColumnElement, base))
-def compute_up(t, lhs, rhs, **kwargs):
+@compute_up.register(BinOp, (ColumnElement, base), ColumnElement)
+@compute_up.register(BinOp, ColumnElement, (ColumnElement, base))
+def binop_sql(t, lhs, rhs, **kwargs):
     return t.op(lhs, rhs)
 
 
-@dispatch(BinOp, (ColumnElement, base), ColumnElement)
-def compute_up(t, lhs, rhs, **kwargs):
-    return t.op(lhs, rhs)
+@dispatch(FloorDiv, ColumnElement)
+def compute_up(t, data, **kwargs):
+    if isinstance(t.lhs, Expr):
+        return sa.func.floor(data / t.rhs)
+    else:
+        return sa.func.floor(t.rhs / data)
+
+
+@compute_up.register(FloorDiv, (ColumnElement, base), ColumnElement)
+@compute_up.register(FloorDiv, ColumnElement, (ColumnElement, base))
+def binop_sql(t, lhs, rhs, **kwargs):
+    return sa.func.floor(lhs / rhs)
 
 
 @dispatch(UnaryOp, ColumnElement)
@@ -120,7 +154,7 @@ def compute_up(t, s, **kwargs):
 def compute_up(t, s, scope=None, **kwargs):
     ns = dict((t._child[col.name], col) for col in s.inner_columns)
     predicate = compute(t.predicate, toolz.merge(ns, scope),
-                            optimize=False, post_compute=False)
+                        optimize=False, post_compute=False)
     if isinstance(predicate, Select):
         predicate = list(list(predicate.columns)[0].base_columns)[0]
     return s.where(predicate)
@@ -130,7 +164,7 @@ def compute_up(t, s, scope=None, **kwargs):
 def compute_up(t, s, scope=None, **kwargs):
     ns = dict((t._child[col.name], lower_column(col)) for col in s.columns)
     predicate = compute(t.predicate, toolz.merge(ns, scope),
-                            optimize=False, post_compute=False)
+                        optimize=False, post_compute=False)
     if isinstance(predicate, Select):
         predicate = list(list(predicate.columns)[0].base_columns)[0]
     try:
@@ -142,11 +176,11 @@ def compute_up(t, s, scope=None, **kwargs):
 def select(s):
     """ Permissive SQL select
 
-    Idempotent sqlalchemy.select
+    Idempotent sa.select
 
     Wraps input in list if neccessary
     """
-    if not isinstance(s, sqlalchemy.sql.Select):
+    if not isinstance(s, sa.sql.Select):
         if not isinstance(s, (tuple, list)):
             s = [s]
         s = sa.select(s)
@@ -158,6 +192,7 @@ def computefull(t, s):
 
 table_names = ('table_%d' % i for i in itertools.count(1))
 
+
 def name(sel):
     """ Name of a selectable """
     if hasattr(sel, 'name'):
@@ -167,6 +202,7 @@ def name(sel):
             return name(sel.froms[0])
     return next(table_names)
 
+
 @dispatch(Select, Select)
 def _join_selectables(a, b, condition=None, **kwargs):
     return a.join(b, condition, **kwargs)
@@ -178,7 +214,7 @@ def _join_selectables(a, b, condition=None, **kwargs):
         raise MDNotImplementedError()
 
     return a.replace_selectable(a.froms[0],
-                a.froms[0].join(b, condition, **kwargs))
+                                a.froms[0].join(b, condition, **kwargs))
 
 
 @dispatch(ClauseElement, Select)
@@ -186,7 +222,8 @@ def _join_selectables(a, b, condition=None, **kwargs):
     if len(b.froms) > 1:
         raise MDNotImplementedError()
     return b.replace_selectable(b.froms[0],
-                a.join(b.froms[0], condition, **kwargs))
+                                a.join(b.froms[0], condition, **kwargs))
+
 
 @dispatch(ClauseElement, ClauseElement)
 def _join_selectables(a, b, condition=None, **kwargs):
@@ -256,26 +293,28 @@ def compute_up(t, lhs, rhs, **kwargs):
 
     fields = [f.replace('_left', '').replace('_right', '') for f in t.fields]
     columns = [c for c in main_cols if c.name in t._on_left]
-    columns += [c for c in left_cols if c.name in fields
-                                    and c.name not in t._on_left]
-    columns += [c for c in right_cols if c.name in fields
-                                     and c.name not in t._on_right]
+    columns += [c for c in left_cols
+                if c.name in fields and c.name not in t._on_left]
+    columns += [c for c in right_cols
+                if c.name in fields and c.name not in t._on_right]
 
     if isinstance(join, Select):
         return join.with_only_columns(columns)
     else:
-        return sqlalchemy.sql.select(columns, from_obj=join)
+        return sa.select(columns, from_obj=join)
 
 
-names = {mean: 'avg',
-         var: 'variance',
-         std: 'stdev'}
+names = {
+    mean: 'avg'
+}
 
 
 @dispatch((nunique, Reduction), Select)
 def compute_up(t, s, **kwargs):
+    if t.axis != (0,):
+        raise ValueError('axis not equal to 0 not defined for SQL reductions')
     d = dict((t._child[c], list(inner_columns(s))[i])
-            for i, c in enumerate(t._child.fields))
+             for i, c in enumerate(t._child.fields))
     col = compute(t, d, post_compute=False)
 
     s = copy(s)
@@ -283,7 +322,7 @@ def compute_up(t, s, **kwargs):
     return s.with_only_columns([col])
 
 
-@dispatch(Distinct, sqlalchemy.Column)
+@dispatch(Distinct, sa.Column)
 def compute_up(t, s, **kwargs):
     return s.distinct().label(t._name)
 
@@ -295,14 +334,28 @@ def compute_up(t, s, **kwargs):
 
 @dispatch(Reduction, sql.elements.ClauseElement)
 def compute_up(t, s, **kwargs):
+    if t.axis != (0,):
+        raise ValueError('axis not equal to 0 not defined for SQL reductions')
     try:
-        op = getattr(sqlalchemy.sql.functions, t.symbol)
+        op = getattr(sa.sql.functions, t.symbol)
     except AttributeError:
-        symbol = names.get(type(t), t.symbol)
-        op = getattr(sqlalchemy.sql.func, symbol)
-    result = op(s)
+        op = getattr(sa.sql.func, names.get(type(t), t.symbol))
+    return op(s).label(t._name)
 
-    return result.label(t._name)
+
+prefixes = {
+    std: 'stddev',
+    var: 'var'
+}
+
+
+@dispatch((std, var), sql.elements.ColumnElement)
+def compute_up(t, s, **kwargs):
+    if t.axis != (0,):
+        raise ValueError('axis not equal to 0 not defined for SQL reductions')
+    funcname = 'samp' if t.unbiased else 'pop'
+    full_funcname = '%s_%s' % (prefixes[type(t)], funcname)
+    return getattr(sa.func, full_funcname)(s).label(t._name)
 
 
 @dispatch(count, Selectable)
@@ -310,14 +363,16 @@ def compute_up(t, s, **kwargs):
     return compute_up(t, select(s), **kwargs)
 
 
-@dispatch(count, sqlalchemy.Table)
+@dispatch(count, sa.Table)
 def compute_up(t, s, **kwargs):
+    if t.axis != (0,):
+        raise ValueError('axis not equal to 0 not defined for SQL reductions')
     try:
         c = list(s.primary_key)[0]
     except IndexError:
         c = list(s.columns)[0]
 
-    return sqlalchemy.sql.functions.count(c)
+    return sa.func.count(c)
 
 
 @dispatch(nelements, (Select, ClauseElement))
@@ -327,6 +382,8 @@ def compute_up(t, s, **kwargs):
 
 @dispatch(count, Select)
 def compute_up(t, s, **kwargs):
+    if t.axis != (0,):
+        raise ValueError('axis not equal to 0 not defined for SQL reductions')
     al = next(aliases)
     try:
         s2 = s.alias(al)
@@ -335,49 +392,62 @@ def compute_up(t, s, **kwargs):
         s2 = s.alias(al)
         col = list(s2.columns)[0]
 
-    result = sqlalchemy.sql.functions.count(col)
+    result = sa.func.count(col)
 
     return select([list(inner_columns(result))[0].label(t._name)])
 
 
-@dispatch(nunique, sqlalchemy.Column)
+@dispatch(nunique, sa.Column)
 def compute_up(t, s, **kwargs):
-    return sqlalchemy.sql.functions.count(s.distinct())
+    if t.axis != (0,):
+        raise ValueError('axis not equal to 0 not defined for SQL reductions')
+    return sa.func.count(s.distinct())
 
 
-@dispatch(Distinct, sqlalchemy.Table)
+@dispatch(nunique, Selectable)
+def compute_up(expr, data, **kwargs):
+    return select(data).distinct().alias(next(aliases)).count()
+
+
+@dispatch(Distinct, Selectable)
 def compute_up(t, s, **kwargs):
     return select(s).distinct()
 
-@dispatch(By, sqlalchemy.Column)
+
+@dispatch(By, sa.Column)
 def compute_up(t, s, **kwargs):
     grouper = lower_column(s)
-    if isinstance(t.apply, Reduction):
-        reductions = [compute(t.apply, {t._child: s}, post_compute=False)]
-    elif isinstance(t.apply, Summary):
-        reductions = [compute(val, {t._child: s}, post_compute=None).label(name)
-                for val, name in zip(t.apply.values, t.apply.fields)]
+    scope = {t._child: s}
+    app = t.apply
+    if isinstance(app, Reduction):
+        reductions = [compute(app, scope, post_compute=False)]
+    elif isinstance(app, Summary):
+        reductions = [compute(val, scope, post_compute=None).label(name)
+                      for val, name in zip(app.values, app.fields)]
 
-    return sqlalchemy.select([grouper] + reductions).group_by(grouper)
+    return sa.select([grouper] + reductions).group_by(grouper)
 
 
 @dispatch(By, ClauseElement)
-def compute_up(t, s, **kwargs):
-    if (isinstance(t.grouper, (Field, Projection)) or
-            t.grouper is t._child):
-        # d = dict((c.name, c) for c in inner_columns(s))
-        # grouper = [d[col] for col in t.grouper.fields]
-        grouper = [lower_column(s.c.get(col)) for col in t.grouper.fields]
+def compute_up(expr, data, **kwargs):
+    if (isinstance(expr.grouper, (Field, Projection)) or
+            expr.grouper is expr._child):
+        grouper = [lower_column(data.c.get(col))
+                   for col in expr.grouper.fields]
+    elif isinstance(expr.grouper, DateTime):
+        grouper = [compute(expr.grouper, data, post_compute=False)]
     else:
-        raise ValueError("Grouper must be a projection, got %s"
-                                  % t.grouper)
-    if isinstance(t.apply, Reduction):
-        reductions = [compute(t.apply, {t._child: s}, post_compute=False)]
-    elif isinstance(t.apply, Summary):
-        reductions = [compute(val, {t._child: s}, post_compute=None).label(name)
-                for val, name in zip(t.apply.values, t.apply.fields)]
+        raise ValueError("Grouper must be a projection, field or "
+                         "DateTime expression, got %s" % expr.grouper)
+    app = expr.apply
+    scope = {expr._child: data}
+    if isinstance(expr.apply, Reduction):
+        reductions = [compute(app, scope, post_compute=False)]
+    elif isinstance(expr.apply, Summary):
+        reductions = [compute(val, scope, post_compute=False).label(name)
+                      for val, name in zip(app.values, app.fields)]
 
-    return sqlalchemy.select(grouper + reductions).group_by(*grouper)
+    return sa.select(grouper + reductions).group_by(*grouper)
 
 
 def lower_column(col):
@@ -416,12 +486,13 @@ def lower_column(col):
 
 aliases = ('alias_%d' % i for i in itertools.count(1))
 
+
 @toolz.memoize
 def alias_it(s):
     """ Alias a Selectable if it has a group by clause """
     if (hasattr(s, '_group_by_clause') and
-        s._group_by_clause is not None and
-        len(s._group_by_clause)):
+            s._group_by_clause is not None and
+            len(s._group_by_clause)):
         return s.alias(next(aliases))
     else:
         return s
@@ -429,10 +500,9 @@ def alias_it(s):
 
 @dispatch(By, Select)
 def compute_up(t, s, **kwargs):
-    if not (isinstance(t.grouper, (Field, Projection))
-            or t.grouper is t._child):
-        raise ValueError("Grouper must be a projection, got %s"
-                                  % t.grouper)
+    if not (isinstance(t.grouper, (Field, Projection, DateTime)) or
+            t.grouper is t._child):
+        raise ValueError("Grouper must be a projection, got %s" % t.grouper)
 
     s = alias_it(s)
 
@@ -442,29 +512,37 @@ def compute_up(t, s, **kwargs):
     elif isinstance(t.apply, Summary):
         reduction = compute(t.apply, {t._child: s}, post_compute=False)
 
-    # d = dict((c.name, c) for c in inner_columns(s))
-    # grouper = [d[col] for col in t.grouper.fields]
     grouper = [lower_column(s.c.get(col)) for col in t.grouper.fields]
 
-    s2 = reduction.group_by(*grouper)
+    grouper_columns = pipe(grouper, map(get_inner_columns), concat)
+    reduction_columns = pipe(reduction.inner_columns, map(get_inner_columns),
+                             concat)
+    columns = list(unique(chain(grouper_columns, reduction_columns)))
+    if (not isinstance(s, sa.sql.selectable.Alias) or
+            (hasattr(s, 'froms') and isinstance(s.froms[0],
+                                                sa.sql.selectable.Join))):
+        assert len(s.froms) == 1, 'only a single FROM clause supported for now'
+        from_obj = s.froms[0]
+    else:
+        from_obj = None
 
-    for g in grouper:
-        s2.append_column(g)
-
-    cols = list(s2.inner_columns)
-    cols = cols[-len(grouper):] + cols[:-len(grouper)]
-    return s2.with_only_columns(cols)
+    return sa.select(columns=columns,
+                     whereclause=getattr(s, 'element', s)._whereclause,
+                     from_obj=from_obj,
+                     distinct=getattr(s, 'element', s)._distinct,
+                     group_by=grouper,
+                     having=None,  # TODO: we don't have an expression for this
+                     limit=getattr(s, 'element', s)._limit,
+                     offset=getattr(s, 'element', s)._offset,
+                     order_by=get_clause(getattr(s, 'element', s), 'order_by'))
 
 
 @dispatch(Sort, ClauseElement)
 def compute_up(t, s, **kwargs):
-    if isinstance(t.key, (tuple, list)):
-        raise ValueError("Multi-column sort not yet implemented")
     s = select(s)
-    col = lower_column(getattr(s.c, t.key))
-    if not t.ascending:
-        col = sqlalchemy.desc(col)
-    return s.order_by(col)
+    direction = sa.asc if t.ascending else sa.desc
+    cols = [direction(lower_column(getattr(s.c, c))) for c in listpack(t.key)]
+    return s.order_by(*cols)
 
 
 @dispatch(Head, Select)
@@ -494,21 +572,104 @@ def compute_up(t, s, **kwargs):
     return select(columns)
 
 
-@dispatch(Merge, Selectable)
-def compute_up(t, s, **kwargs):
-    subexpression = common_subexpression(*t.children)
-    children = [compute(child, {subexpression: s}, post_compute=False)
-                 for child in t.children]
-    return select(children)
+@dispatch(sa.sql.FromClause)
+def get_inner_columns(sel):
+    return list(map(lower_column, sel.c.values()))
+
+
+@dispatch(sa.sql.elements.ColumnElement)
+def get_inner_columns(c):
+    return [c]
+
+
+@dispatch(sa.sql.selectable.ScalarSelect)
+def get_inner_columns(sel):
+    inner_columns = list(sel.inner_columns)
+    assert len(inner_columns) == 1, 'ScalarSelect should have only ONE column'
+    return list(map(lower_column, sel.inner_columns))
+
+
+@dispatch(sa.sql.functions.Function)
+def get_inner_columns(f):
+    assert len(f.clauses.clauses) == 1, \
+        'only single argument functions allowed'
+    unique_columns = unique(concat(map(get_inner_columns, f.clauses)))
+    lowered = [x.label(getattr(x, 'name', None)) for x in unique_columns]
+    return list(map(getattr(sa.func, f.name), lowered))
+
+
+@dispatch(sa.sql.elements.Label)
+def get_inner_columns(label):
+    """
+    Notes
+    -----
+    This should only ever return a list of length 1
+
+    This is because we need to turn ScalarSelects into an actual column
+    """
+    name = label.name
+    inner_columns = get_inner_columns(label.element)
+    assert len(inner_columns) == 1
+    return [lower_column(c).label(name) for c in inner_columns]
+
+
+@dispatch(Select)
+def get_all_froms(sel):
+    return list(unique(sel.locate_all_froms()))
+
+
+@dispatch(sa.Table)
+def get_all_froms(t):
+    return [t]
+
+
+@dispatch(sa.sql.elements.ColumnClause)
+def get_all_froms(c):
+    return [c.table]
+
+
+def get_clause(data, kind):
+    # arg SQLAlchemy doesn't allow things like data._group_by_clause or None
+    assert kind == 'order_by' or kind == 'group_by', \
+        'kind must be "order_by" or "group_by"'
+    clause = getattr(data, '_%s_clause' % kind, None)
+    return clause.clauses if clause is not None else None
+
+
+@dispatch(Merge, (Selectable, Select, sa.Column))
+def compute_up(expr, data, **kwargs):
+    # get the common subexpression of all the children in the merge
+    subexpression = common_subexpression(*expr.children)
+
+    # compute each child, including the common subexpression
+    children = [compute(child, {subexpression: data}, post_compute=False)
+                for child in expr.children]
+
+    # Get the original columns from the selection and rip out columns from
+    # Selectables and ScalarSelects
+    columns = list(unique(concat(map(get_inner_columns, children))))
+
+    # we need these getattrs if data is a ColumnClause or Table
+    from_obj = get_all_froms(data)
+    assert len(from_obj) == 1, 'only a single FROM clause supported'
+    return sa.select(columns,
+                     from_obj=from_obj,
+                     whereclause=getattr(data, '_whereclause', None),
+                     distinct=getattr(data, '_distinct', False),
+                     having=getattr(data, '_having', None),
+                     group_by=get_clause(data, 'group_by'),
+                     limit=getattr(data, '_limit', None),
+                     offset=getattr(data, '_offset', None),
+                     order_by=get_clause(data, 'order_by'))
 
 
 @dispatch(Summary, Select)
 def compute_up(t, s, scope=None, **kwargs):
     d = dict((t._child[c], list(inner_columns(s))[i])
-            for i, c in enumerate(t._child.fields))
+             for i, c in enumerate(t._child.fields))
 
     cols = [compute(val, toolz.merge(scope, d), post_compute=None).label(name)
-                for name, val in zip(t.fields, t.values)]
+            for name, val in zip(t.fields, t.values)]
 
     s = copy(s)
     for c in cols:
@@ -520,7 +681,7 @@ def compute_up(t, s, scope=None, **kwargs):
 @dispatch(Summary, ClauseElement)
 def compute_up(t, s, **kwargs):
     return select([compute(value, {t._child: s}, post_compute=None).label(name)
-        for value, name in zip(t.values, t.fields)])
+                   for value, name in zip(t.values, t.fields)])
 
 
 @dispatch(Like, Selectable)
@@ -530,14 +691,38 @@ def compute_up(t, s, **kwargs):
 
 @dispatch(Like, Select)
 def compute_up(t, s, **kwargs):
-    d = dict()
-    for name, pattern in t.patterns.items():
-        for f in s.froms:
-            if f.c.has_key(name):
-                d[f.c.get(name)] = pattern.replace('*', '%')
+    items = [(f.c.get(name), pattern.replace('*', '%'))
+             for name, pattern in t.patterns.items()
+             for f in s.froms if name in f.c]
+    return s.where(reduce(and_, [key.like(pattern) for key, pattern in items]))
 
-    return s.where(reduce(and_,
-                          [key.like(pattern) for key, pattern in d.items()]))
+
+string_func_names = {
+    # <blaze function name>: <SQL function name>
+}
+
+
+# TODO: remove if the alternative fix goes into PyHive
+@compiles(sa.sql.functions.Function, 'hive')
+def compile_char_length_on_hive(element, compiler, **kwargs):
+    assert len(element.clauses) == 1, \
+        'char_length must have a single clause, got %s' % list(element.clauses)
+    if element.name == 'char_length':
+        return compiler.visit_function(sa.func.length(*element.clauses),
+                                       **kwargs)
+    return compiler.visit_function(element, **kwargs)
+
+
+@dispatch(strlen, ColumnElement)
+def compute_up(expr, data, **kwargs):
+    return sa.sql.functions.char_length(data).label(expr._name)
+
+
+@dispatch(UnaryStringFunction, ColumnElement)
+def compute_up(expr, data, **kwargs):
+    func_name = type(expr).__name__
+    func_name = string_func_names.get(func_name, func_name)
+    return getattr(sa.sql.func, func_name)(data).label(expr._name)
 
 
 @toolz.memoize
@@ -552,11 +737,29 @@ def table_of_engine(engine, name):
     return table_of_metadata(metadata, name)
 
 
-@dispatch(Field, sqlalchemy.engine.Engine)
+@dispatch(Field, sa.engine.Engine)
 def compute_up(expr, data, **kwargs):
     return table_of_engine(data, expr._name)
 
 
+@dispatch(DateTime, (ClauseElement, sa.sql.elements.ColumnElement))
+def compute_up(expr, data, **kwargs):
+    return sa.extract(expr.attr, data).label(expr._name)
+
+
+@compiles(sa.sql.elements.Extract, 'hive')
+def hive_extract_to_date_function(element, compiler, **kwargs):
+    func = getattr(sa.func, element.field)(element.expr)
+    return compiler.visit_function(func, **kwargs)
+
+
+@compiles(sa.sql.elements.Extract, 'mssql')
+def mssql_extract_to_datepart(element, compiler, **kwargs):
+    func = sa.func.datepart(sa.sql.expression.column(element.field),
+                            element.expr)
+    return compiler.visit_function(func, **kwargs)
+
+
 def engine_of(x):
     if isinstance(x, Engine):
         return x
@@ -567,15 +770,12 @@ def engine_of(x):
     raise NotImplementedError("Can't deterimine engine of %s" % x)
 
 
-from ..expr.broadcast import broadcast_collect
-
-
 @dispatch(Expr, sa.sql.elements.ClauseElement)
 def optimize(expr, _):
     return broadcast_collect(expr)
 
 
-@dispatch(Field, sqlalchemy.MetaData)
+@dispatch(Field, sa.MetaData)
 def compute_up(expr, data, **kwargs):
     return table_of_metadata(data, expr._name)
 
@@ -583,3 +783,8 @@ def compute_up(expr, data, **kwargs):
 @dispatch(Expr, sa.sql.elements.ClauseElement)
 def post_compute(_, s, **kwargs):
     return select(s)
+
+
+@dispatch(IsIn, ColumnElement)
+def compute_up(expr, data, **kwargs):
+    return data.in_(expr._keys)
