@@ -410,3 +410,31 @@ def test_join():
     expr = join(t, c, 'name')
     result = compute(expr, {t: x, c: cities})
     assert (b'Alice', 1, 100, 'LA') in into(list, result)
+
+
+def test_query_with_strings():
+    b = np.array([('a', 1), ('b', 2), ('c', 3)],
+                 dtype=[('x', 'S1'), ('y', 'i4')])
+
+    s = symbol('s', discover(b))
+    assert compute(s[s.x == b'b'], b).tolist() == [(b'b', 2)]
+
+
+@pytest.mark.parametrize('keys', [['a'], list('bc')])
+def test_isin(keys):
+    b = np.array([('a', 1), ('b', 2), ('c', 3), ('a', 4), ('c', 5), ('b', 6)],
+                 dtype=[('x', 'S1'), ('y', 'i4')])
+
+    s = symbol('s', discover(b))
+    result = compute(s.x.isin(keys), b)
+    expected = np.in1d(b['x'], keys)
+    np.testing.assert_array_equal(result, expected)
+
+
+def test_nunique_recarray():
+    b = np.array([('a', 1), ('b', 2), ('c', 3), ('a', 4), ('c', 5), ('b', 6),
+                  ('a', 1), ('b', 2)],
+                 dtype=[('x', 'S1'), ('y', 'i4')])
+    s = symbol('s', discover(b))
+    expr = s.nunique()
+    assert compute(expr, b) == len(np.unique(b))
