@@ -536,6 +536,13 @@ def test_like():
     tm.assert_frame_equal(result, expected)
 
 
+def test_strlen():
+    expr = t.name.strlen()
+    expected = pd.Series([5, 3, 5], name='name')
+    result = compute(expr, df).reset_index(drop=True)
+    tm.assert_series_equal(expected, result)
+
+
 def test_rowwise_by():
     f = lambda _, id, name: id + len(name)
     expr = by(t.map(f, 'int'), total=t.amount.sum())
@@ -647,3 +654,17 @@ def test_by_with_complex_summary():
     result = compute(expr, df)
     assert list(result.columns) == expr.fields
     assert list(result.total) == [150 + 4 - 1, 200 + 2 - 1]
+
+
+@pytest.mark.parametrize('keys', [[1], [2, 3]])
+def test_isin(keys):
+    expr = t[t.id.isin(keys)]
+    result = compute(expr, df)
+    expected = df.loc[df.id.isin(keys)]
+    tm.assert_frame_equal(result, expected)
+
+
+def test_nunique_table():
+    expr = t.nunique()
+    result = compute(expr, df)
+    assert result == len(df.drop_duplicates())
