@@ -10,10 +10,14 @@ import os
 import itertools
 import shutil
 
+from py4j.protocol import Py4JJavaError
 import numpy as np
 import pandas as pd
 from blaze import compute, symbol, into, by, sin, exp, cos, tan, join
-from blaze.compute.spark import SparkDataFrame
+try:
+    from pyspark.sql import DataFrame as SparkDataFrame
+except ImportError:
+    from pyspark.sql import SchemaRDD as SparkDataFrame
 from pyspark import HiveContext, SQLContext
 from pyspark.sql import Row, SchemaRDD
 from odo import odo, discover
@@ -24,10 +28,23 @@ data = [['Alice', 100.0, 1],
         ['Bob', 200.0, 2],
         ['Alice', 50.0, 3]]
 
+date_data = []
+
+for attr in ('YearBegin', 'MonthBegin', 'Day', 'Hour', 'Minute', 'Second'):
+    rng = pd.date_range(start='now', periods=len(data),
+                        freq=getattr(pd.datetools, attr)()).values
+    date_data += list(zip(np.random.choice(['Alice', 'Bob', 'Joe', 'Lester'],
+                                           size=len(data)),
+                          np.random.rand(len(data)) * 100,
+                          np.random.randint(100, size=3),
+                          rng))
+
+
 cities_data = [['Alice', 'NYC'],
                ['Bob', 'Boston']]
 
 df = pd.DataFrame(data, columns=['name', 'amount', 'id'])
+date_df = pd.DataFrame(date_data, columns=['name', 'amount', 'id', 'ds'])
 cities_df = pd.DataFrame(cities_data, columns=['name', 'city'])
 
 
@@ -72,16 +89,31 @@ def cities(sc):
         yield parts.map(lambda person: Row(name=person[0], city=person[1]))
 
 
+@pytest.yield_fixture(scope='module')
+def date_people(sc):
+    with tmpfile('.txt') as fn:
+        date_df.to_csv(fn, header=False, index=False)
+        raw = sc.textFile(fn)
+        parts = raw.map(lambda line: line.split(','))
+        yield parts.map(lambda person: Row(name=person[0],
+                                           amount=float(person[1]),
+                                           id=int(person[2]),
+                                           ds=pd.Timestamp(person[3]).to_pydatetime()))
+
+
 @pytest.fixture(scope='module')
-def ctx(sql, people, cities):
+def ctx(sql, people, cities, date_people):
     try:
         sql.registerDataFrameAsTable(sql.createDataFrame(people), 't')
         sql.cacheTable('t')
         sql.registerDataFrameAsTable(sql.createDataFrame(cities), 's')
         sql.cacheTable('s')
+        sql.registerDataFrameAsTable(sql.createDataFrame(date_people), 'dates')
+        sql.cacheTable('dates')
     except AttributeError:
         sql.inferSchema(people).registerTempTable('t')
         sql.inferSchema(cities).registerTempTable('s')
+        sql.inferSchema(date_people).registerTempTable('dates')
     return sql
 
 
@@ -223,7 +255,8 @@ def test_math(ctx, db, func):
 
 
 @pytest.mark.parametrize(['field', 'ascending'],
-                         itertools.product(['name', 'id'], [True, False]))
+                         itertools.product(['name', 'id', ['name', 'amount']],
+                                           [True, False]))
 def test_sort(ctx, db, field, ascending):
     expr = db.t.sort(field, ascending=ascending)
     result = compute(expr, ctx)
@@ -286,3 +319,53 @@ def test_by_non_native_ops(ctx, db):
     expected = compute(expr, {db: {'t': df}})
     assert list(map(set, into(list, result))) == list(map(set, into(list,
                                                                     expected)))
+
+
+@pytest.mark.xfail(not hasattr(pyspark.sql, 'types'),
+                   reason=('length string function not available without '
+                           'HiveContext'),
+                   raises=py4j.protocol.Py4JJavaError)
+def test_strlen(ctx, db):
+    expr = db.t.name.strlen()
+    result = odo(compute(expr, ctx), pd.Series)
+    expected = compute(expr, {db: {'t': df}})
+    assert result.name == 'name'
+    assert expected.name == 'name'
+    assert odo(result, set) == odo(expected, set)
+
+
+date_attrs = [pytest.mark.xfail(not hasattr(pyspark.sql, 'types'),
+                                attr,
+                                raises=Py4JJavaError,
+                                reason=('date attribute %r not supported '
+                                        'without hive') % attr)
+              for attr in ['year', 'month', 'day', 'hour', 'minute', 'second']]
+
+date_attrs += [pytest.mark.xfail(attr,
+                                 raises=Py4JJavaError,
+                                 reason=('Hive does not support date '
+                                         'attribute %r') % attr)
+               for attr in ['millisecond', 'microsecond']]
+
+
+@pytest.mark.parametrize('attr', date_attrs)
+def test_by_with_date(ctx, db, attr):
+    expr = by(getattr(db.dates.ds, attr),
+              mean=db.dates.amount.mean())
+    result = odo(compute(expr, ctx), set)
+    expected = odo(compute(expr, {db: {'dates': date_df}}), set)
+    assert result == expected
+
+
+@pytest.mark.parametrize('keys', [[1], [1, 2]])
+def test_isin(ctx, db, keys):
+    expr = db.t[db.t.id.isin(keys)]
+    result = odo(compute(expr, ctx), set)
+    expected = odo(compute(expr, {db: {'t': df}}), set)
+    assert (set(map(frozenset, odo(result, list))) ==
+            set(map(frozenset, odo(expected, list))))
+
+
+def test_nunique_spark_dataframe(ctx, db):
+    assert (odo(compute(db.t.nunique(), ctx), int) ==
+            ctx.table('t').distinct().count())
