@@ -1,22 +1,51 @@
 from __future__ import absolute_import, division, print_function
 
-import pytest
-sqlalchemy = pytest.importorskip('sqlalchemy')
-sa = sqlalchemy
-
 import re
+from distutils.version import LooseVersion
+
+import pytest
+
+sa = pytest.importorskip('sqlalchemy')
 
 import datashape
+from odo import into, resource
+from pandas import DataFrame
+from toolz import unique
 
 from blaze.compute.sql import (compute, computefull, select, lower_column,
                                compute_up)
-from blaze.expr import *
+from blaze.expr import (symbol, discover, transform, summary, by, sin, join,
+                        floor, cos, merge, nunique, mean, sum, count, exp)
 from blaze.compatibility import xfail
-from toolz import unique
-from pandas import DataFrame
-from odo import into, resource
 from blaze.utils import tmpfile
 
+
+@pytest.fixture(scope='module')
+def data():
+    # make the engine
+    engine = sa.create_engine('sqlite:///:memory:')
+    metadata = sa.MetaData(engine)
+
+    # name table
+    name = sa.Table('name', metadata,
+                    sa.Column('id', sa.Integer),
+                    sa.Column('name', sa.String),
+                    )
+    name.create()
+
+    # city table
+    city = sa.Table('city', metadata,
+                    sa.Column('id', sa.Integer),
+                    sa.Column('city', sa.String),
+                    sa.Column('country', sa.String),
+                    )
+    city.create()
+
+    s = symbol('s', discover(engine))
+    return {'engine': engine, 'metadata': metadata, 'name': name, 'city': city,
+            's': s}
+
+
 t = symbol('t', 'var * {name: string, amount: int, id: int}')
 
 metadata = sa.MetaData()
@@ -24,22 +53,22 @@ metadata = sa.MetaData()
 s = sa.Table('accounts', metadata,
              sa.Column('name', sa.String),
              sa.Column('amount', sa.Integer),
-             sa.Column('id', sa.Integer, primary_key=True),
-             )
+             sa.Column('id', sa.Integer, primary_key=True))
 
-tbig = symbol('tbig', 'var * {name: string, sex: string[1], amount: int, id: int}')
+tbig = symbol('tbig',
+              'var * {name: string, sex: string[1], amount: int, id: int}')
 
 sbig = sa.Table('accountsbig', metadata,
-             sa.Column('name', sa.String),
-             sa.Column('sex', sa.String),
-             sa.Column('amount', sa.Integer),
-             sa.Column('id', sa.Integer, primary_key=True),
-             )
+                sa.Column('name', sa.String),
+                sa.Column('sex', sa.String),
+                sa.Column('amount', sa.Integer),
+                sa.Column('id', sa.Integer, primary_key=True))
+
 
 def normalize(s):
-    s2 = ' '.join(s.strip().split()).lower().replace('_', '')
-    s3 = re.sub('alias\d*', 'alias', s2)
-    return s3
+    s = ' '.join(s.strip().split()).lower()
+    s = re.sub(r'(alias)_?\d*', r'\1', s)
+    return re.sub(r'__([A-Za-z_][A-Za-z_0-9]*)', r'\1', s)
 
 
 def test_table():
@@ -129,7 +158,7 @@ def test_join():
     assert normalize(str(result)) == normalize("""
     SELECT amounts.name, amounts.amount, ids.id
     FROM amounts JOIN ids ON amounts.name = ids.name
-    ORDER BY amounts.amount""")
+    ORDER BY amounts.amount ASC""")
 
 
 
@@ -204,12 +233,24 @@ def test_unary_op():
     assert str(compute(exp(t['amount']), s, post_compute=False)) == \
             str(sa.func.exp(s.c.amount))
 
-
-def test_unary_op():
     assert str(compute(-t['amount'], s, post_compute=False)) == \
             str(-s.c.amount)
 
 
+@pytest.mark.parametrize('unbiased', [True, False])
+def test_std(unbiased):
+    assert str(compute(t.amount.std(unbiased=unbiased), s, post_compute=False)) == \
+        str(getattr(sa.func,
+                    'stddev_%s' % ('samp' if unbiased else 'pop'))(s.c.amount))
+
+
+@pytest.mark.parametrize('unbiased', [True, False])
+def test_var(unbiased):
+    assert str(compute(t.amount.var(unbiased=unbiased), s, post_compute=False)) == \
+        str(getattr(sa.func,
+                    'var_%s' % ('samp' if unbiased else 'pop'))(s.c.amount))
+
+
 def test_reductions():
     assert str(compute(sum(t['amount']), s, post_compute=False)) == \
             str(sa.sql.functions.sum(s.c.amount))
@@ -221,6 +262,17 @@ def test_reductions():
     assert 'amount_sum' == compute(sum(t['amount']), s, post_compute=False).name
 
 
+def test_reduction_with_invalid_axis_argument():
+    with pytest.raises(ValueError):
+        compute(t.amount.mean(axis=1))
+
+    with pytest.raises(ValueError):
+        compute(t.count(axis=1))
+
+    with pytest.raises(ValueError):
+        compute(t[['amount', 'id']].count(axis=1))
+
+
 def test_nelements():
     rhs = str(compute(t.count(), s))
     assert str(compute(t.nelements(), s)) == rhs
@@ -232,7 +284,7 @@ def test_nelements():
 @pytest.mark.xfail(raises=Exception, reason="We don't support axis=1 for"
                    " Record datashapes")
 def test_nelements_axis_1():
-    assert compute(nelements(t, axis=1), s) == len(s.columns)
+    assert compute(t.nelements(axis=1), s) == len(s.columns)
 
 
 def test_count_on_table():
@@ -256,8 +308,9 @@ def test_count_on_table():
               FROM accounts
               WHERE accounts.amount > :amount_1) as alias"""))
 
+
 def test_distinct():
-    result = str(compute(Distinct(t['amount']), s, post_compute=False))
+    result = str(compute(t['amount'].distinct(), s, post_compute=False))
 
     assert 'distinct' in result.lower()
     assert 'amount' in result.lower()
@@ -281,10 +334,18 @@ def test_nunique():
     assert 'amount' in result.lower()
 
 
+def test_nunique_table():
+    result = normalize(str(computefull(t.nunique(), s)))
+    expected = normalize("""SELECT count(alias.id) AS tbl_row_count
+FROM (SELECT DISTINCT accounts.name AS name, accounts.amount AS amount, accounts.id AS id
+FROM accounts) as alias""")
+    assert result == expected
+
+
 @xfail(reason="Fails because SQLAlchemy doesn't seem to know binary reductions")
 def test_binary_reductions():
     assert str(compute(any(t['amount'] > 150), s)) == \
-            str(sqlalchemy.sql.functions.any(s.c.amount > 150))
+        str(sa.sql.functions.any(s.c.amount > 150))
 
 
 def test_by():
@@ -394,22 +455,30 @@ def test_join_projection():
 
 def test_sort():
     assert str(compute(t.sort('amount'), s)) == \
-            str(select(s).order_by(s.c.amount))
+            str(select(s).order_by(sa.asc(s.c.amount)))
 
     assert str(compute(t.sort('amount', ascending=False), s)) == \
-            str(select(s).order_by(sqlalchemy.desc(s.c.amount)))
+            str(select(s).order_by(sa.desc(s.c.amount)))
+
+
+def test_multicolumn_sort():
+    assert str(compute(t.sort(['amount', 'id']), s)) == \
+        str(select(s).order_by(sa.asc(s.c.amount), sa.asc(s.c.id)))
+
+    assert str(compute(t.sort(['amount', 'id'], ascending=False), s)) == \
+        str(select(s).order_by(sa.desc(s.c.amount), sa.desc(s.c.id)))
 
 
 def test_sort_on_distinct():
     assert normalize(str(compute(t.amount.sort(), s))) == normalize("""
             SELECT accounts.amount
             FROM accounts
-            ORDER BY accounts.amount""")
+            ORDER BY accounts.amount ASC""")
 
     assert normalize(str(compute(t.amount.distinct().sort(), s))) == normalize("""
             SELECT DISTINCT accounts.amount as amount
             FROM accounts
-            ORDER BY amount""")
+            ORDER BY amount ASC""")
 
 
 
@@ -429,18 +498,6 @@ def test_relabel():
     assert str(result) == str(expected)
 
 
-def test_merge():
-    col = (t['amount'] * 2).label('new')
-
-    expr = merge(t['name'], col)
-
-    result = str(compute(expr, s))
-
-    assert 'amount * ' in result
-    assert 'FROM accounts' in result
-    assert 'SELECT accounts.name' in result
-    assert 'new' in result
-
 def test_projection_of_selection():
     print(compute(t[t['amount'] < 0][['name', 'amount']], s))
     assert len(str(compute(t[t['amount'] < 0], s))) > \
@@ -610,10 +667,19 @@ def test_like():
     FROM accounts
     WHERE accounts.name LIKE :name_1""")
 
+
+def test_strlen():
+    expr = t.name.strlen()
+    result = str(compute(expr, s))
+    expected = "SELECT char_length(accounts.name) as name FROM accounts"
+    assert normalize(result) == normalize(expected)
+
+
 def test_columnwise_on_complex_selection():
-    assert normalize(str(select(compute(t[t.amount > 0].amount + 1, s)))) == \
+    result = str(select(compute(t[t.amount > 0].amount + 1, s)))
+    assert normalize(result) == \
             normalize("""
-    SELECT accounts.amount + :amount_1 AS anon_1
+    SELECT accounts.amount + :amount_1 AS amount
     FROM accounts
     WHERE accounts.amount > :amount_2
     """)
@@ -628,7 +694,7 @@ def test_reductions_on_complex_selections():
 
 
 def test_clean_summary_by_where():
-    t2 = t[t.id ==1]
+    t2 = t[t.id == 1]
     expr = by(t2.name, sum=t2.amount.sum(), count=t2.amount.count())
     result = compute(expr, s)
 
@@ -663,8 +729,6 @@ def test_join_complex_clean():
              sa.Column('country', sa.String),
              )
 
-    sel = select(name).where(name.c.id > 10)
-
     tname = symbol('name', discover(name))
     tcity = symbol('city', discover(city))
 
@@ -742,8 +806,6 @@ def test_lower_column():
     tname = symbol('name', discover(name))
     tcity = symbol('city', discover(city))
 
-    ns = {tname: name, tcity: city}
-
     assert lower_column(name.c.id) is name.c.id
     assert lower_column(select(name).c.id) is name.c.id
 
@@ -820,68 +882,39 @@ def test_join_on_same_table():
     """)
 
 
-def test_field_access_on_engines():
-    engine = sa.create_engine('sqlite:///:memory:')
-    metadata = sa.MetaData(engine)
-    name = sa.Table('name', metadata,
-             sa.Column('id', sa.Integer),
-             sa.Column('name', sa.String),
-             )
-    name.create()
-
-    city = sa.Table('city', metadata,
-             sa.Column('id', sa.Integer),
-             sa.Column('city', sa.String),
-             sa.Column('country', sa.String),
-             )
-    city.create()
-
-    s = symbol('s', discover(engine))
+def test_field_access_on_engines(data):
+    s, engine = data['s'], data['engine']
     result = compute_up(s.city, engine)
     assert isinstance(result, sa.Table)
     assert result.name == 'city'
 
 
-def test_computation_directly_on_sqlalchemy_Tables():
-    engine = sa.create_engine('sqlite:///:memory:')
-    metadata = sa.MetaData(engine)
-    name = sa.Table('name', metadata,
-             sa.Column('id', sa.Integer),
-             sa.Column('name', sa.String),
-             )
-    name.create()
-
+def test_computation_directly_on_sqlalchemy_Tables(data):
+    name = data['name']
     s = symbol('s', discover(name))
     result = into(list, compute(s.id + 1, name))
     assert not isinstance(result, sa.sql.Selectable)
     assert list(result) == []
 
 
-def test_computation_directly_on_metadata():
-    engine = sa.create_engine('sqlite:///:memory:')
-    metadata = sa.MetaData(engine)
-    name = sa.Table('name', metadata,
-             sa.Column('id', sa.Integer),
-             sa.Column('name', sa.String),
-             )
-    name.create()
-
+def test_computation_directly_on_metadata(data):
+    metadata = data['metadata']
+    name = data['name']
     s = symbol('s', discover(metadata))
-
     result = compute(s.name, {s: metadata}, post_compute=False)
     assert result == name
 
 
 sql_bank = sa.Table('bank', sa.MetaData(),
-                 sa.Column('id', sa.Integer),
-                 sa.Column('name', sa.String),
-                 sa.Column('amount', sa.Integer))
+                    sa.Column('id', sa.Integer),
+                    sa.Column('name', sa.String),
+                    sa.Column('amount', sa.Integer))
 sql_cities = sa.Table('cities', sa.MetaData(),
-                   sa.Column('name', sa.String),
-                   sa.Column('city', sa.String))
+                      sa.Column('name', sa.String),
+                      sa.Column('city', sa.String))
 
-bank = Symbol('bank', discover(sql_bank))
-cities = Symbol('cities', discover(sql_cities))
+bank = symbol('bank', discover(sql_bank))
+cities = symbol('cities', discover(sql_cities))
 
 
 def test_aliased_views_with_two_group_bys():
@@ -1016,8 +1049,8 @@ def test_aliased_views_with_computation():
     sql_aaa = metadata.tables['aaa']
     sql_bbb = metadata.tables['bbb']
 
-    L = Symbol('aaa', discover(df_aaa))
-    R = Symbol('bbb', discover(df_bbb))
+    L = symbol('aaa', discover(df_aaa))
+    R = symbol('bbb', discover(df_bbb))
 
     expr = join(by(L.x, y_total=L.y.sum()),
                 R)
@@ -1091,7 +1124,7 @@ def test_join_count():
           WHERE t1.x > ?) as alias
           """
     expected2 = """
-    SELECT count(alias2.x) AS __count
+    SELECT count(alias2.x) AS count
     FROM (SELECT alias1.x AS x, alias1.y AS y, t2.b AS b
           FROM (SELECT t1.x AS x, t1.y AS y
                 FROM t1
@@ -1103,6 +1136,121 @@ def test_join_count():
             normalize(str(result)) == normalize(expected2))
 
 
+def test_transform_where():
+    t2 = t[t.id == 1]
+    expr = transform(t2, abs_amt=abs(t2.amount), sine=sin(t2.id))
+    result = compute(expr, s)
+
+    expected = """SELECT
+        accounts.name,
+        accounts.amount,
+        accounts.id,
+        abs(accounts.amount) as abs_amt,
+        sin(accounts.id) as sine
+    FROM accounts
+    WHERE accounts.id = :id_1
+    """
+
+    assert normalize(str(result)) == normalize(expected)
+
+
+def test_merge():
+    col = (t['amount'] * 2).label('new')
+
+    expr = merge(t['name'], col)
+
+    result = str(compute(expr, s))
+
+    assert 'amount * ' in result
+    assert 'FROM accounts' in result
+    assert 'SELECT accounts.name' in result
+    assert 'new' in result
+
+
+def test_merge_where():
+    t2 = t[t.id == 1]
+    expr = merge(t2[['amount', 'name']], t2.id)
+    result = compute(expr, s)
+    expected = normalize("""SELECT
+        accounts.amount,
+        accounts.name,
+        accounts.id
+    FROM accounts
+    WHERE accounts.id = :id_1
+    """)
+    assert normalize(str(result)) == expected
+
+
+def test_transform_filter_by_single_column():
+    t2 = t[t.amount < 0]
+    tr = transform(t2, abs_amt=abs(t2.amount), sine=sin(t2.id))
+    expr = by(tr.name, avg_amt=tr.abs_amt.mean())
+    result = compute(expr, s)
+    expected = normalize("""SELECT
+        accounts.name,
+        avg(abs(accounts.amount)) AS avg_amt
+    FROM accounts
+    WHERE accounts.amount < :amount_1
+    GROUP BY accounts.name
+    """)
+    assert normalize(str(result)) == expected
+
+
+def test_transform_filter_by_multiple_columns():
+    t2 = t[t.amount < 0]
+    tr = transform(t2, abs_amt=abs(t2.amount), sine=sin(t2.id))
+    expr = by(tr.name, avg_amt=tr.abs_amt.mean(), sum_sine=tr.sine.sum())
+    result = compute(expr, s)
+    expected = normalize("""SELECT
+        accounts.name,
+        avg(abs(accounts.amount)) AS avg_amt,
+        sum(sin(accounts.id)) AS sum_sine
+    FROM accounts
+    WHERE accounts.amount < :amount_1
+    GROUP BY accounts.name
+    """)
+    assert normalize(str(result)) == expected
+
+
+def test_transform_filter_by_different_order():
+    t2 = transform(t, abs_amt=abs(t.amount), sine=sin(t.id))
+    tr = t2[t2.amount < 0]
+    expr = by(tr.name,
+              avg_amt=tr.abs_amt.mean(),
+              avg_sine=tr.sine.sum() / tr.sine.count())
+    result = compute(expr, s)
+    expected = normalize("""SELECT
+        accounts.name,
+        avg(abs(accounts.amount)) AS avg_amt,
+        sum(sin(accounts.id)) / count(sin(accounts.id)) AS avg_sine
+    FROM accounts
+    WHERE accounts.amount < :amount_1
+    GROUP BY accounts.name
+    """)
+    assert normalize(str(result)) == expected
+
+
+@pytest.mark.xfail(raises=ValueError,
+                   reason='Need to allow Merge expressions to be groupers')
+def test_transform_filter_by_projection():
+    t2 = transform(t, abs_amt=abs(t.amount), sine=sin(t.id))
+    tr = t2[t2.amount < 0]
+    expr = by(tr[['name', 'id']],
+              avg_amt=tr.abs_amt.mean(),
+              avg_sine=tr.sine.sum() / tr.sine.count())
+    result = compute(expr, s)
+    expected = normalize("""SELECT
+        accounts.name,
+        accounts.id,
+        avg(abs(accounts.amount)) AS avg_amt,
+        sum(sin(accounts.id)) / count(sin(accounts.id)) AS avg_sine
+    FROM accounts
+    WHERE accounts.amount < :amount_1
+    GROUP BY accounts.name, accounts.id
+    """)
+    assert normalize(str(result)) == expected
+
+
 def test_merge_compute():
     data = [(1, 'Alice', 100),
             (2, 'Bob', 200),
@@ -1175,3 +1323,114 @@ def test_no_extraneous_join():
                       FROM operation) AS alias2
                 ON alias1.operation = alias2.name
     """)
+
+
+def test_math():
+    result = compute(sin(t.amount), s)
+    assert normalize(str(result)) == normalize("""
+            SELECT sin(accounts.amount) as amount
+            FROM accounts""")
+
+    result = compute(floor(t.amount), s)
+    assert normalize(str(result)) == normalize("""
+            SELECT floor(accounts.amount) as amount
+            FROM accounts""")
+
+    result = compute(t.amount // 2, s)
+    assert normalize(str(result)) == normalize("""
+            SELECT floor(accounts.amount / :amount_1) AS amount
+            FROM accounts""")
+
+
+def test_transform_order():
+    r = transform(t, sin_amount=sin(t.amount), cos_id=cos(t.id))
+    result = compute(r, s)
+    expected = """SELECT
+        accounts.name,
+        accounts.amount,
+        accounts.id,
+        cos(accounts.id) as cos_id,
+        sin(accounts.amount) as sin_amount
+    FROM accounts
+    """
+    assert normalize(str(result)) == normalize(expected)
+
+
+def test_isin():
+    result = t[t.name.isin(['foo', 'bar'])]
+    result_sql_expr = str(compute(result, s))
+    expected = """
+        SELECT
+            accounts.name,
+            accounts.amount,
+            accounts.id
+        FROM
+            accounts
+        WHERE
+            accounts.name
+        IN
+            (:name_1,
+            :name_2)
+    """
+    assert normalize(result_sql_expr) == normalize(expected)
+
+
+@pytest.mark.skipif(LooseVersion(sa.__version__) >= '1.0.0',
+                    reason="SQLAlchemy generates different code in >= 1.0.0")
+def test_date_grouper_repeats_not_one_point_oh():
+    columns = [sa.Column('amount', sa.REAL),
+               sa.Column('ds', sa.TIMESTAMP)]
+    data = sa.Table('t', sa.MetaData(), *columns)
+    t = symbol('t', discover(data))
+    expr = by(t.ds.year, avg_amt=t.amount.mean())
+    result = str(compute(expr, data))
+
+    # FYI spark sql isn't able to parse this correctly
+    expected = """SELECT
+        EXTRACT(year FROM t.ds) as ds_year,
+        AVG(t.amount) as avg_amt
+    FROM t
+    GROUP BY EXTRACT(year FROM t.ds)
+    """
+    assert normalize(result) == normalize(expected)
+
+
+@pytest.mark.skipif(LooseVersion(sa.__version__) < '1.0.0',
+                    reason="SQLAlchemy generates different code in < 1.0.0")
+def test_date_grouper_repeats():
+    columns = [sa.Column('amount', sa.REAL),
+               sa.Column('ds', sa.TIMESTAMP)]
+    data = sa.Table('t', sa.MetaData(), *columns)
+    t = symbol('t', discover(data))
+    expr = by(t.ds.year, avg_amt=t.amount.mean())
+    result = str(compute(expr, data))
+
+    # FYI spark sql isn't able to parse this correctly
+    expected = """SELECT
+        EXTRACT(year FROM t.ds) as ds_year,
+        AVG(t.amount) as avg_amt
+    FROM t
+    GROUP BY ds_year
+    """
+    assert normalize(result) == normalize(expected)
+
+
+def test_transform_then_project_single_column():
+    expr = transform(t, foo=t.id + 1)[['foo', 'id']]
+    result = normalize(str(compute(expr, s)))
+    expected = normalize("""SELECT
+        accounts.id + :id_1 as foo,
+        accounts.id
+    FROM accounts""")
+    assert result == expected
+
+
+def test_transform_then_project():
+    proj = ['foo', 'id']
+    expr = transform(t, foo=t.id + 1)[proj]
+    result = normalize(str(compute(expr, s)))
+    expected = normalize("""SELECT
+        accounts.id + :id_1 as foo,
+        accounts.id
+    FROM accounts""")
+    assert result == expected
