@@ -2,42 +2,54 @@ from __future__ import absolute_import, division, print_function
 
 import datashape
 import datetime
-from datashape import discover, Tuple, Record, DataShape, var
-from datashape.predicates import iscollection, isscalar, isrecord
-from pandas import DataFrame, Series
+import operator
 import itertools
-from functools import reduce
-import numpy as np
 import warnings
+
 from collections import Iterator
+from functools import reduce
+
+from datashape import discover, Tuple, Record, DataShape, var
+from datashape.predicates import iscollection, isscalar, isrecord, istabular
+
+from pandas import DataFrame, Series
+
+import numpy as np
+
+from odo import resource, odo
+from odo.utils import ignoring
 
 from .expr import Expr, Symbol, ndim
-from .utils import ignoring
 from .dispatch import dispatch
-from odo import into, resource
 from .compatibility import _strtypes
 
+
 __all__ = ['Data', 'Table', 'into', 'to_html']
 
+
 names = ('_%d' % i for i in itertools.count(1))
-
-
 not_an_iterator = []
 
-try:
+
+with ignoring(ImportError):
     import bcolz
     not_an_iterator.append(bcolz.carray)
-except ImportError:
-    pass
-try:
+
+
+with ignoring(ImportError):
     import pymongo
     not_an_iterator.append(pymongo.collection.Collection)
-except ImportError:
-    pass
+    not_an_iterator.append(pymongo.database.Database)
 
 
 def Data(data, dshape=None, name=None, fields=None, columns=None, schema=None,
          **kwargs):
+    if columns:
+        raise ValueError("columns argument deprecated, use fields instead")
+    if schema and dshape:
+        raise ValueError("Please specify one of schema= or dshape= keyword"
+                         " arguments")
+
     sub_uri = ''
     if isinstance(data, _strtypes):
         if '::' in data:
@@ -47,14 +59,6 @@ def Data(data, dshape=None, name=None, fields=None, columns=None, schema=None,
     if (isinstance(data, Iterator) and
             not isinstance(data, tuple(not_an_iterator))):
         data = tuple(data)
-    if columns:
-        warnings.warn("columns kwarg deprecated.  Use fields instead",
-                      DeprecationWarning)
-    if columns and not fields:
-        fields = columns
-    if schema and dshape:
-        raise ValueError("Please specify one of schema= or dshape= keyword"
-                         " arguments")
     if schema and not dshape:
         dshape = var * schema
     if dshape and isinstance(dshape, _strtypes):
@@ -71,6 +75,18 @@ def Data(data, dshape=None, name=None, fields=None, columns=None, schema=None,
             schema = Record(list(zip(fields, types)))
             dshape = DataShape(*(dshape.shape[:-1] + (schema,)))
         elif isrecord(dshape.measure) and fields:
+            ds = discover(data)
+            assert isrecord(ds.measure)
+            names = ds.measure.names
+            if names != fields:
+                raise ValueError('data column names %s\n'
+                                 '\tnot equal to fields parameter %s,\n'
+                                 '\tuse Data(data).relabel(%s) to rename '
+                                 'fields' % (names,
+                                             fields,
+                                             ', '.join('%s=%r' % (k, v)
+                                                       for k, v in
+                                                       zip(names, fields))))
             types = dshape.measure.types
             schema = Record(list(zip(fields, types)))
             dshape = DataShape(*(dshape.shape + (schema,)))
@@ -87,7 +103,7 @@ def Data(data, dshape=None, name=None, fields=None, columns=None, schema=None,
 
 
 class InteractiveSymbol(Symbol):
-    """ Interactive data
+    """Interactive data.
 
     The ``Data`` object presents a familiar view onto a variety of forms of
     data.  This user-level object provides an interactive experience to using
@@ -95,26 +111,23 @@ class InteractiveSymbol(Symbol):
 
     Parameters
     ----------
-
-    data: anything
+    data : object
         Any type with ``discover`` and ``compute`` implementations
-    fields: list of strings - optional
+    fields : list, optional
         Field or column names, will be inferred from datasource if possible
-    dshape: string or DataShape - optional
-        Datashape describing input data
-    name: string - optional
-        A name for the table
+    dshape : str or DataShape, optional
+        DataShape describing input data
+    name : str, optional
+        A name for the data.
 
     Examples
     --------
-
     >>> t = Data([(1, 'Alice', 100),
     ...           (2, 'Bob', -200),
     ...           (3, 'Charlie', 300),
     ...           (4, 'Denis', 400),
     ...           (5, 'Edith', -500)],
     ...          fields=['id', 'name', 'balance'])
-
     >>> t[t.balance < 0].name
         name
     0    Bob
@@ -134,13 +147,16 @@ class InteractiveSymbol(Symbol):
 
     @property
     def _args(self):
-        return (id(self.data), self.dshape, self._name)
+        return id(self.data), self.dshape, self._name
 
     def __setstate__(self, state):
         for slot, arg in zip(self.__slots__, state):
             setattr(self, slot, arg)
 
 
+Data.__doc__ = InteractiveSymbol.__doc__
+
+
 def Table(*args, **kwargs):
     """ Deprecated, see Data instead """
     warnings.warn("Table is deprecated, use Data instead",
@@ -172,11 +188,11 @@ def concrete_head(expr, n=10):
     head = expr.head(n + 1)
 
     if not iscollection(expr.dshape):
-        return into(object, head)
+        return odo(head, object)
     elif isrecord(expr.dshape.measure):
-        return into(DataFrame, head)
+        return odo(head, DataFrame)
     else:
-        df = into(DataFrame, head)
+        df = odo(head, DataFrame)
         df.columns = [expr._name]
         return df
     result = compute(head)
@@ -184,9 +200,9 @@ def concrete_head(expr, n=10):
     if len(result) == 0:
         return DataFrame(columns=expr.fields)
     if isrecord(expr.dshape.measure):
-        return into(DataFrame, result, dshape=expr.dshape)
+        return odo(result, DataFrame, dshape=expr.dshape)
     else:
-        df = into(DataFrame, result, dshape=expr.dshape)
+        df = odo(result, DataFrame, dshape=expr.dshape)
         df.columns = [expr._name]
         return df
 
@@ -197,7 +213,6 @@ def repr_tables(expr, n=10):
     if isinstance(result, (DataFrame, Series)):
         s = repr(result)
         if len(result) > 10:
-            result = result[:10]
             s = '\n'.join(s.split('\n')[:-1]) + '\n...'
         return s
     else:
@@ -209,7 +224,7 @@ def numel(shape):
         return None
     if not shape:
         return 1
-    return reduce(lambda x, y: x * y, shape, 1)
+    return reduce(operator.mul, shape, 1)
 
 
 def short_dshape(ds, nlines=5):
@@ -223,8 +238,8 @@ def short_dshape(ds, nlines=5):
 def coerce_to(typ, x):
     try:
         return typ(x)
-    except:
-        return into(typ, x)
+    except TypeError:
+        return odo(x, typ)
 
 
 def coerce_scalar(result, dshape):
@@ -252,9 +267,9 @@ def expr_repr(expr, n=10):
         return repr(coerce_scalar(compute(expr), str(expr.dshape)))
 
     # Tables
-    with ignoring(Exception):
-        if ndim(expr) == 1:
-            return repr_tables(expr, 10)
+    if (ndim(expr) == 1 and (istabular(expr.dshape) or
+                             isscalar(expr.dshape.measure))):
+        return repr_tables(expr, 10)
 
     # Smallish arrays
     if ndim(expr) >= 2 and numel(expr.shape) and numel(expr.shape) < 1000000:
@@ -317,7 +332,7 @@ Expr.__len__ = table_length
 
 def intonumpy(data, dtype=None, **kwargs):
     # TODO: Don't ignore other kwargs like copy
-    result = into(np.ndarray, data)
+    result = odo(data, np.ndarray)
     if dtype and result.dtype != dtype:
         result = result.astype(dtype)
     return result
@@ -328,7 +343,7 @@ def convert_base(typ, x):
     try:
         return typ(x)
     except:
-        return typ(into(typ, x))
+        return typ(odo(x, typ))
 
 Expr.__array__ = intonumpy
 Expr.__int__ = lambda x: convert_base(int, x)
