@@ -8,9 +8,11 @@ from blaze import compute, Data, by, into, discover
 from blaze.expr import Expr, symbol, Field
 from blaze.dispatch import dispatch
 from blaze.server import Server
-from blaze.server.index import parse_index, emit_index
 from blaze.server.client import Client, resource
 
+from blaze.server.tests.fixtures import make_app_context_fixture
+
+
 df = DataFrame([['Alice', 100], ['Bob', 200]],
                columns=['name', 'amount'])
 
@@ -27,7 +29,10 @@ from blaze.server import client
 client.requests = test # OMG monkey patching
 
 
-def test_client():
+app_context = make_app_context_fixture(server)
+
+
+def test_client(app_context):
     c = Client('localhost:6363')
     assert str(discover(c)) == str(discover(data))
 
@@ -40,7 +45,7 @@ def test_client():
     assert compute(t.accounts.name, c) == ['Alice', 'Bob']
 
 
-def test_expr_client_interactive():
+def test_expr_client_interactive(app_context):
     c = Client('localhost:6363')
     t = Data(c)
 
@@ -49,7 +54,8 @@ def test_expr_client_interactive():
                                                   max=t.accounts.amount.max())))
             == set([('Alice', 100, 100), ('Bob', 200, 200)]))
 
-def test_compute_client_with_multiple_datasets():
+
+def test_compute_client_with_multiple_datasets(app_context):
     c = resource('blaze://localhost:6363')
     s = symbol('s', discover(c))
 
@@ -57,23 +63,23 @@ def test_compute_client_with_multiple_datasets():
                     {s: c}) == 600
 
 
-def test_resource():
+def test_resource(app_context):
     c = resource('blaze://localhost:6363')
     assert isinstance(c, Client)
     assert str(discover(c)) == str(discover(data))
 
 
-def test_resource_default_port():
+def test_resource_default_port(app_context):
     ec = resource('blaze://localhost')
     assert str(discover(ec)) == str(discover(data))
 
 
-def test_resource_non_default_port():
+def test_resource_non_default_port(app_context):
     ec = resource('blaze://localhost:6364')
     assert ec.url == 'http://localhost:6364'
 
 
-def test_resource_all_in_one():
+def test_resource_all_in_one(app_context):
     ec = resource('blaze://localhost:6363')
     assert str(discover(ec)) == str(discover(data))
 
@@ -91,13 +97,13 @@ def compute_up(expr, data, **kwargs):
     return data
 
 
-def test_custom_expressions():
+def test_custom_expressions(app_context):
     ec = Client('localhost:6363')
     t = symbol('t', discover(ec))
 
     assert list(map(tuple, compute(CustomExpr(t.accounts), ec))) == into(list, df)
 
 
-def test_client_dataset():
+def test_client_dataset(app_context):
     d = Data('blaze://localhost::accounts')
     assert list(map(tuple, into(list, d))) == into(list, df)
